#!/usr/bin/bash

export LANG=en_US.UTF-8

trap ctrl_c INT

function ctrl_c() {
    echo "*** Trapped CTRL-C ***"
    exit 1
}

_stage=$1
_stage_lock="/root/initial/tasks/${_stage}.done"
_centos_release=$(/bin/rpm --eval='%centos_ver')

stage1() {
    check_stage
    while ! which puppet &> /dev/null; do
        echo "puppet not installed"
        dnf -y install nf-puppet3
        sleep 5
    done
    sed -i "s#ETHDEV.*#ETHDEV $(ip route show default 0.0.0.0/0 | grep -Po '(?<=dev )\S+')#" /etc/wwwacct.conf
    timedatectl set-timezone "${TIME_ZONE}"
    timedatectl set-local-rtc 0
    timedatectl set-ntp true
    service cpanel stop
    mkdir -vp /opt/eig_linux/etc/puppetfeatures
    touch /opt/eig_linux/etc/puppetfeatures/{datacycle,remi_repo,pssh42}
    rm -vf {/var/cpanel/globalcache/cpanel.cache,/root/.cpanel/datastore/MOUNTS_CACHE__proc_mounts}
    if [[ -f /root/initial/conf/hal_local_data.yaml ]]; then
      _pupp_cmd="imeout -s 9 5m puppet agent -t --server hensonm.hostgator.com --ca_server hensonca.hostgator.com --tags foreman_registration"
    else
      _pupp_cmd="FACTER_foreman_organization=${BRAND} FACTER_foreman_location=${LOCATION} timeout -s 9 5m puppet agent -t --server hensonm.hostgator.com --ca_server hensonca.hostgator.com --tags foreman_registration"
    fi
    ${_pupp_cmd}
    while [[ $? =~ ^(1|137)$ ]]; do
        sleep 5
	${_pupp_cmd}
    done
    puppet agent --disable 'provisioning in progress'
    echo "$(date) :: Completed ${_stage}"
    touch "${_stage_lock}"
}

stage2() {
    check_stage
    puppet agent --enable

    ## applying repos,puppet config
    timeout -s 9 5m puppet agent -t --server hensonm.hostgator.com --ca_server hensonca.hostgator.com --tags=yum_repos,puppet_agent,resolvers
    while [[ $? =~ ^(1|137)$ ]]; do
        sleep 5
         timeout -s 9 5m puppet agent -t --server hensonm.hostgator.com --ca_server hensonca.hostgator.com --tags=yum_repos,puppet_agent,resolvers
    done

    ## this will normally fail if prov process failed to configure server in correct hostgroup.
    timeout -s 9 5m puppet agent -t --tags=yum_repos,puppet_agent,resolvers
    while [[ $? =~ ^(1|137)$ ]]; do
        sleep 5
         timeout -s 9 5m puppet agent -t --tags=yum_repos,puppet_agent,resolvers
    done

    puppet agent --disable 'provisioning in progress'
    rpm --import https://rpms.remirepo.net/enterprise/9/RPM-GPG-KEY-remi
    dnf clean all -y --enablerepo=*
    dnf makecache
    echo "$(date) :: Completed ${_stage}"
    touch "${_stage_lock}"
}

stage3() {
    check_stage
    service ipaliases start
    if [[ ! -z ${cloud_name+x} ]]; then
      /usr/local/cpanel/scripts/build_cpnat
      md5sum /root/.accesshash | awk {'print $1'} | tr -d '\n' > /root/.accesshash.md5
    fi
    sed -i '/\(upcp --cron\|optimizefs\|update_db_cache\)/d' /var/spool/cron/root
    if [ "${PRODUCT}" != 'owp' ]; then
      ### Add postgres for BH ESOP-2655
      if [ "${BRAND}" == "BLUEHOST" ]; then
          /scripts/installpostgres --yes
      fi
      wget -q -O /root/install_softac.sh --no-check-certificate http://files.softaculous.com/install.sh
      while [ $? -ne 0 ]; do
        sleep 5
        wget -q -O /root/install_softac.sh --no-check-certificate http://files.softaculous.com/install.sh
      done
      chmod 755 /root/install_softac.sh
      /root/install_softac.sh &>> /root/initial/softaculous_install.log
      rm -f /root/install_softac.sh
      touch /opt/eig_linux/etc/puppetfeatures/softaculous
      chmod 4555  /usr/local/cpanel/whostmgr/docroot/cgi/softaculous/bin/soft
    fi

    /usr/local/cpanel/bin/checkallsslcerts
    whmapi1 php_set_default_accounts_to_fpm default_accounts_to_fpm 0
    whmapi1 disable_cpgreylist
    puppet agent --enable

    if ! systemctl is-active mysql &> /dev/null; then
      if ! systemctl start mysql &> /dev/null; then
        echo "MySQL failed to start"
        exit 1
      fi
    fi

    timeout -s 9 600s puppet agent -t
    while [ $? -ne 0 ]; do
        if ! systemctl is-active mysql &> /dev/null; then
          if ! systemctl start mysql &> /dev/null; then
            echo "MySQL failed to start"
            exit 1
          fi
        fi
        sleep 5
        timeout -s 9 600s puppet agent -t
    done
    if ! systemctl is-active mysql &> /dev/null; then
      if ! systemctl start mysql &> /dev/null; then
        echo "MySQL failed to start"
        exit 1
      fi
    fi

    if rpm -q --quiet postgresql; then
        /opt/eig_linux/bin-legacy/fixpostgres
    fi
    rm -vf /var/cpanel/ssl/*-{CRTINFO,NOT_AFTER,CN,SIGNATURE_CHAIN_VERIFIED,DOMAINS}
    whmapi1 set_autossl_metadata metadata_json='{"clobber_externally_signed":1, "notify_autossl_expiry":0, "notify_autossl_expiry_coverage":0, "notify_autossl_renewal":0, "notify_autossl_renewal_coverage":0, "notify_autossl_renewal_coverage_reduced":0, "notify_autossl_renewal_uncovered_domains":0, "notify_autossl_expiry_coverage_user":0, "notify_autossl_expiry_user":0, "notify_autossl_renewal_coverage_reduced_user":0, "notify_autossl_renewal_coverage_user":0, "notify_autossl_renewal_uncovered_domains_user":0 }'
    /opt/hgmods/multi_shared.pl
    touch /var/cpanel/diya_allow
    rm -vf /admin/MIRRORING.FROM
    chkconfig fastmail off
    /scripts/fixquotas

    if [[ -z ${prov_env+x} ]]; then
      ### custapi randomly drops requests causing it to fail. Adding this as a work around
      /opt/hgmods/hal_accesshash.pl
      while [ $? -eq 1 ]; do
        sleep 5
        /opt/hgmods/hal_accesshash.pl
      done
    fi
    sed -i -r '/\/(home[1-9]?)/s/[0-9]$/0/' /etc/fstab
    echo "$(date) :: Completed ${_stage}"
    touch "${_stage_lock}"
}

stage4() {
    check_stage
    #cp_update
    if [ -f /etc/cpanel/ea4/is_ea4 ]; then
        _phpver=ea-php82
	### TODO figure out better way to manage these long term
        dnf install libc-client -y || exit 1
        dnf groupinstall alt-php74 -y || exit 1
	###
        /usr/local/bin/ea_install_profile --install /etc/cpanel/ea4/profiles/custom/ea4_eig_profile.json || exit 1
        _php_vers=( $(/bin/awk -F: '/lsapi/ {print $1}' <(/usr/local/cpanel/bin/rebuild_phpconf --available)) )
        /usr/local/cpanel/bin/rebuild_phpconf --default=${_phpver}$(printf -- ' --%s=lsapi' "${_php_vers[@]}") || exit 1
    fi
    /scripts/upcp --sync

    timeout -s 9 5m puppet agent -t
    while [ $? -ne 0 ]; do
        if ! systemctl is-active mysql &> /dev/null; then
          if ! systemctl start mysql &> /dev/null; then
            echo "MySQL failed to start"
            exit 1
          fi
        fi
        sleep 5
        timeout -s 9 5m puppet agent -t
    done
    if ! systemctl is-active mysql &> /dev/null; then
      if ! systemctl start mysql &> /dev/null; then
        echo "MySQL failed to start"
        exit 1
      fi
    fi

    if ! mysql -BNe 'select User from user where User = "collectd"' -Dmysql | grep -q collectd; then
      _passwd=$(awk '/Password/ {print $2}' /etc/collectd.conf| sed 's/"//g')
      while ! mysql -BNe "CREATE USER 'collectd'@'localhost' IDENTIFIED BY '${_passwd}'; GRANT USAGE, PROCESS, REPLICATION CLIENT ON *.* TO collectd@localhost;"; do
        echo "${HOSTNAME} fail to add grant"
        sleep 5
      done
      echo "${HOSTNAME} added grant"
    fi
    rm -vf /var/cpanel/conf/apache/local
    /scripts/rebuildhttpdconf
    while ! httpd -t; do
      sleep 5
      /scripts/rebuildhttpdconf
    done

    if rpm -q --quiet python3-perf &> /dev/null; then
      dnf remove -y python3-perf
    fi
    while ! dnf -y update; do
      echo "Pending DNF updates"
      sleep 5
      dnf -y update
    done
    if ! rpm -q --quiet tuned &> /dev/null; then
      dnf install -y tuned
      systemctl --now enable tuned
      tuned-adm profile virtual-guest
    fi

    if [[ -s /etc/motd ]]; then
      > /etc/motd
    fi

    echo "$(date) :: Completed ${_stage}"
    touch "${_stage_lock}"
}

stage5() {
    check_stage

    ### right now in OCI when kernel updates via cloud-init default kernel entry is not getting updated correctly
    nf_kernel=$(\ls /boot/vmlinuz-5.14.0-*nf* | tail -1)
    if [[ ${nf_kernel} != $(grubby --default-kernel) ]]; then
      echo "Setting default kernel to ${nf_kernel}"
      grubby --set-default=${nf_kernel}
    fi

    if _var=$(rpm -qa kernel-core | grep -v nf | paste -sd " " | grep [a-z]); then
      if [[ $(uname -r) =~ nf ]]; then
        dnf remove -y ${_var}
      else
        echo "Can't remove extra kernel-core RPMs without first being booted into NF kernel"
        exit 1
      fi
    fi

    dnf -y remove initial-scripts-${BRAND}
    if [[ "$(dmidecode -s chassis-asset-tag)" != 'OracleCloud.com' ]]; then
      echo "Rebooting in 2 Minutes"
      systemctl --now enable atd
      echo "shutdown -rf now" | at now + 2 min
    fi
    echo "$(date) :: Completed ${_stage}"
    touch "${_stage_lock}"
}

check_stage() {
    if [ -f "${_stage_lock}" ]; then
        exit 0
    fi
    echo "$(date) :: Running ${_stage}"
}

${_stage}
